# -*- coding: utf-8 -*-

"""Top-level package for CM4AI Generate Hierarchy."""

__author__ = 'Clara Hu'
__email__ = 'mhu@ucsd.edu'
__version__ = '0.1.0a2'
__repo_url__ = 'https://github.com/idekerlab/cellmaps_generate_hierarchy'
__description__ = 'A tool to generate hierarchies from protein to protein interaction networks'
